"""
EMIL EMAIL CLIENT
WRITTEN BY ANDREAS PRLIC <andorsch@gmx.at>
this document is protected by the GPL
"""

import Globals
import OFS
from OFS import SimpleItem

from Globals import DTMLFile
from Globals import Persistent
import Acquisition                	# Various acquisition types. 
import AccessControl.Role         	# Zope Security management.

from pop_connection import pop_connection
from email import Email

import random




class EmailFolder(OFS.Folder.Folder) :

    """ A EMIL email client product """

    meta_type = 'EMIL_Email_Client'

    manage_options = ((OFS.ObjectManager.ObjectManager.manage_options[0],)+
		      (	{'label': 'Messages',   'action': 'index_html',},
			{'label': 'Write_Email', 'action': 'write_mail_html',},
			{'label': 'Check_Email', 'action': 'check_mail_html',},		      
			{'label': 'Properties',  'action': 'manage_editForm',},) +
		      AccessControl.Role.RoleManager.manage_options+
		      OFS.SimpleItem.Item.manage_options+
		      OFS.FindSupport.FindSupport.manage_options
		      )

    
    __ac_permissions__=(
	('View management screens',
	 ['manage_editForm','manage_editAction'],
	 ('Owner',)),	
	 ('View',
	  ['index_html','check_mail_html',],
	  ('Owner')),     
        )     

    # The web pages that shows content. Put your own in the dtml folder.

    # Used to view content of the object:
    index_html       = DTMLFile('dtml/index_html'      , globals())
    manage_editForm  = DTMLFile('dtml/manage_editForm' , globals())
    check_mail_html  = DTMLFile('dtml/check_mail_html' , globals())
    write_mail_html  = DTMLFile('dtml/write_mail_html' , globals())
    folder_view_html = DTMLFile('dtml/folder_view_html' , globals())
    
    def __init__ (self, id, title, server, username, password, get_new, realname='',reply_to='' ) :
	""" inits the EMIL with default values """
	
	self.id       = id
	self.title    = title
	self.server   = server
	self.username = username
	self.password = password
	self.realname = realname
	self.reply_to = reply_to
	
	# uidl keeps track of which emails have been dowloaded from server
	# it is a dictionary of email uid ( see email class)	
	# if no mails are on server, uid can be reset.
	self.uidl = {}

	# get new is a flag to download all emails, or only the new ones
	if get_new == '' :
	    self.get_new =  0
	else :
	    self.get_new = 1

	self.manage_addFolder('Inbox','incoming email folder')
	self.manage_addFolder('Outbox','outgoing email folder')

	# store their names
	self.incoming_folder = getattr(self,'Inbox')
	self.outgoing_folder = getattr(self,'Outbox')
	
	
    def __call__(self):
	""" note we initially dont have an index_html of our own,
	 just in the style of OFS.Folder we would end up
	 acquiring one
	"""
	return self.index_html.__call__()
	
    def manage_editAction(self, title, server, username, password, get_new='', realname='',reply_to='' , REQUEST=None,RESPONSE=None) :
	"Changes the product values"

	self.title    = title
	self.server   = server
	self.username = username
	self.password = password
	if get_new == '' :
	    self.get_new =  0
	else :
	    self.get_new = 1

	self.realname = realname
	self.reply_to = reply_to
	
	self._p_changed = 1
	#RESPONSE.redirect('manage_editForm')
	message = "properties have been changed"
	return self.manage_editForm(self, REQUEST,manage_tabs_message=message)

    
    def reset_uidl(self) :
	" reset the uid dictionary, only do this when there are no mails on server... "
	self.uidl = {}


    
    def check_EmailAction(self,REQUEST=None,RESPONSE=None) :
	"checks the email"

	#print "checking_emails"
	pop = pop_connection(server=self.server,user=self.username,password=self.password)
	pop.open()
	
	number_mails, octet =  pop.status()	
	# no mails on server, uidl can be reset
	if number_mails == 0 :
	    self.reset_uidl()
	    message= "no mails on server"
	    return self.manage_main(self, REQUEST,manage_tabs_message=message)

	#status, uidl, octet = pop.uidl()
	#print status, uidl, octet
	messages = pop.get_messages(get_new=self.get_new,known_uidl=self.uidl)
	pop.close()

	txt = ""
	
	c = 0
	for message in messages :
	    txt  = message['list']
	    size = message['size']
	    uid  = message['uid']
	    c = c + 1
	    id = self.create_unique_id()
	    
	    #id = "%s" % c 
	    title = '' 
	    self.manage_addEmail(self.incoming_folder,id,title,size,uid)
	    obj = getattr(self.incoming_folder,id)
	    obj.from_server_list(txt)

	    self.uidl[uid] = 1

	if c > 1:
	    message = "downloaded %s new mails (on server: %s)" % (c,number_mails)
	elif c == 1 :
	    message = "downloaded 1 mail (on server: %s)" % number_mails
	else :
	    if number_mails == 1 :
		message = "no new mails on server (%s old mail)" % number_mails
	    else :
		message = "no new mails on server (%s old mails)" % number_mails
	
	return self.index_html(self, REQUEST,manage_tabs_message=message)

    def email_sendAction(self,To='',From='',Subject='',Body='',REQUEST=None) :
	" send an email "
	id     = self.create_unique_id()
	title  = ''
	uid    = None
	size   = None
	status ='queued'
	
	self.manage_addEmail(self.outgoing_folder,id,title,size,uid)
	obj = getattr(self.outgoing_folder,id)
	obj.set_values(From=From, To=To,Subject=Subject,Body=Body,status=status,)
	
	# assume that MailHost is called MailHost :
	self.MailHost.simple_send(mto=To,mfrom=From,subject=Subject,body=Body)

	obj.set_status('sent')
	message = "message successfully sent to %s" % To
	return self.index_html(self, REQUEST,manage_tabs_message=message)

   
    def create_unique_id(self) :
	" create unique id, perhaps somebody knwos a  better solution ?, mail me!"
	id = str(int(self.ZopeTime()))
	id =  id + str(random.random())
	return id 

    def manage_addEmail(self,folder,id,title,size,uid) :
	" add an email to a folder"	
	folder._setObject(id,Email(id,title,size,uid))
	

    def get_email_stats(self):
	"return # email / folder "
	ret = []
	for folder in self.objectValues('Folder') :
	    
	    email_lst = folder.objectValues('Email')
	    unread_count = 0
	    for email in email_lst :
		if email.status == 'unread' :
		    unread_count = unread_count + 1
	    lst = []
	    fo = ('folder',folder.id)
	    to = ('total',len(email_lst))
	    un = ('unread',unread_count)
	    lst.append(fo)
	    lst.append(to)
	    lst.append(un)
	   
	    ret.append(lst)
	return ret

    def get_emails(self,Folder) :
	" return list of emails "
	"return emails in current folder"
	path = getattr(self,Folder)
	return path.objectValues('Email')

    
 
# constructor pages. Only used when the product is added to a folder:

# Get user id from this form :
manage_addEMILForm 	= DTMLFile('dtml/manage_addEMILForm', globals()) 

    
def manage_addEMILAction(self,
			 id='EMIL',
			 title='Title here',
			 server='',
			 username='',
			 password='',
			 get_new='',
			 realname='',
			 reply_to='',			 
			 REQUEST=None):
    "Add a EMIL"
    self._setObject(id, EmailFolder(id, title, server, username, password,get_new,realname,reply_to))
   
    if REQUEST is not None:
	return self.manage_main(self, REQUEST)
    


	

